@@ -29,20 +29,27 @@ The module currently contains the following classes:
 
 * ``DiffusionMaps``: Class for multi point data-based dimensionality reduction.
 
+* ``POD``: Class for dimension reduction of solution snapshots.
+
 """
 
-from UQpy.Surrogates import Kriging  
-from UQpy.Utilities import * 
-from UQpy.Utilities import _nn_coord
-import scipy as sp
-import numpy as np
-import itertools
-from scipy.interpolate import LinearNDInterpolator
 import copy
+import itertools
+
+import numpy as np
+import matplotlib.pyplot as plt
+from sklearn.preprocessing import normalize
+from UQpy.Utilities import svd
+import functools
 
 import scipy.sparse as sps
 import scipy.sparse.linalg as spsl
 import scipy.spatial.distance as sd
+from scipy.interpolate import LinearNDInterpolator
+
+from UQpy.Surrogates import Kriging
+from UQpy.Utilities import *
+from UQpy.Utilities import _nn_coord
 
 
 ########################################################################################################################
@@ -83,9 +90,7 @@ class Grassmann:
         can pass either a method of a class or a function. For example, if the user wish to
         use `grassmann_distance` to compute the distance, one can use the following command:
 
-        >>> Grassmann(distance_method=Grassmann.grassmann_distance)
-
-         On the other hand, if the user implemented a function
+        On the other hand, if the user implemented a function
         (e.g., `user_distance`) to compute the distance, `distance_method` must assume the following value
         `distance_method = user_distance`, which must be pre-loaded using import. In this regard, the
         function input must contain the first (x0) and second (x1) matrices as arguments (e.g, user_distance(x0,x1))
@@ -102,8 +107,6 @@ class Grassmann:
         For example, if the user wish to use `projection_kernel` to estimate the kernel matrix, one can use the
         following command:
 
-        >>> Grassmann(kernel_method=Grassmann.projection_kernel)
-
         On the other hand, if the user implemented
         a function (e.g., `user_kernel`) to compute the kernel matrix, `kernel_method` must assume the following value
         `kernel_object = user_kernel`, which must be pre-loaded using import. In this regard, the function
@@ -159,6 +162,7 @@ class Grassmann:
         representing a point on the Grassmann manifold.
 
     **Methods:**
+
     """
 
     def __init__(self, distance_method=None, kernel_method=None, interp_object=None, karcher_method=None):
@@ -178,7 +182,7 @@ class Grassmann:
                 raise TypeError('UQpy: A callable kernel object must be provided.')
         
         # Interpolation.
-        skl_str = "<class 'sklearn.gaussian_process.gpr.GaussianProcessRegressor'>"
+        skl_str = "<class 'sklearn.gaussian_process._gpr.GaussianProcessRegressor'>"
         self.skl = str(type(interp_object))==skl_str
         if interp_object is not None:
             if callable(interp_object) or isinstance(interp_object, Kriging) or self.skl:
@@ -217,8 +221,6 @@ class Grassmann:
         method serves to set the manifold and to verify the consistency of the input data. This method can be called
         using the following command:
 
-        >>> Grassmann.manifold(p=p,samples=samples,append_samples=False)
-
         In this case, append_samples is a boolean variable used to define if new sample will replace the previous ones
         or will just get appended (`append_samples=True`).
 
@@ -238,6 +240,7 @@ class Grassmann:
         * **append_samples** (`bool`)
             The attributes are replaced when manifold is called if `append_samples` is False, otherwise the lists are 
             appended.
+
         """
 
         # If manifold called for the first time 
@@ -396,9 +399,6 @@ class Grassmann:
         points on the Grassmann manifold. For example, given the points on the Grassmann manifold one can compute the 
         pairwise distances in the following way:
 
-        >>> Gr = Grassmann(distance_method=Grassmann.grassmann_distance)
-        >>> Gr.distance(points_grassmann=points_grassmann)
-
         **Input:**
 
         * **points_grassmann** (`list` or `NoneType`) 
@@ -418,6 +418,7 @@ class Grassmann:
         * **points_distance_phi** (`list`)
             Pairwise distance of points on the manifold defined by the right singular eigenvectors if `points_grassmann` 
             is not provided.
+
         """
 
         # Show an error message if no distance_object is identified.
@@ -480,6 +481,7 @@ class Grassmann:
 
         * **distance_list** (`list`)
             Pairwise distance.
+
         """
 
         # Check points for type and shape consistency.
@@ -541,6 +543,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Grassmann distance between x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -587,6 +590,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Chordal distance between x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -633,6 +637,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Procrustes distance between x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -679,6 +684,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Projection distance between x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -725,6 +731,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Projection distance between x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -765,9 +772,6 @@ class Grassmann:
         to compute the kernel must be the points on the same manifold; therefore, it is important to ensure that the 
         dimension of all the input matrices are the same. Therefore, the following commands can be used:
 
-        >>> Gr = Grassmann(kernel_method=Grassmann.projection_kernel)
-        >>> Gr.kernel(points_grassmann=points_grassmann)
-
         **Input:**
 
         * **points_grassmann** (`list` or `NoneType`) 
@@ -787,6 +791,7 @@ class Grassmann:
         * **kernel_matrix_phi** (`list`)
             Kernel matrix on the manifold defined by the right singular eigenvectors if `points_grassmann` 
             is not provided.
+
         """
 
         # If points_grassmann is None get the information set using the method manifold.
@@ -852,6 +857,7 @@ class Grassmann:
 
         * **kernel_matrix** (`list`)
             Kernel matrix.
+
         """
 
         # Check points for type and shape consistency.
@@ -923,6 +929,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Kernel value for x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -960,6 +967,7 @@ class Grassmann:
 
         * **distance** (`float`)
             Kernel value for x0 and x1.
+
         """
 
         if not isinstance(x0, list) and not isinstance(x0, np.ndarray):
@@ -1001,6 +1009,7 @@ class Grassmann:
 
         * **points_tan**: (`list`)
             Point on the tangent space.
+
         """
 
         # Show an error message if points_grassmann is not provided.
@@ -1084,6 +1093,7 @@ class Grassmann:
 
         * **points_manifold**: (`list`)
             Point on the tangent space.
+
         """
 
         # Show an error message if points_tangent is not provided.
@@ -1149,10 +1159,6 @@ class Grassmann:
         square distances, defined on the Grassmann manifold, to a given point. The command to compute the Karcher mean
         given a seto of points on the Grassmann manifold is.
 
-        >>> Gr = Grassmann(karcher_method=Grassmann.gradient_descent)
-        >>> Gr.manifold(p=p,samples=samples,append_samples=False)
-        >>> karcher_left, karcher_right = Gr.karcher_mean()
-
         In this case two values are returned corresponding to the ones related to the manifolds defined by the left and
         right singular eigenvectors.
 
@@ -1177,6 +1183,7 @@ class Grassmann:
             
         * **kr_mean_phi** (`list`)
             Karcher mean for right singular eigenvectors if `points_grassmann` is not provided.
+
         """
 
         # Show an error message if karcher_object is not provided.
@@ -1249,6 +1256,7 @@ class Grassmann:
 
         * **mean_element** (`list`)
             Karcher mean.
+
         """
 
         # acc is a boolean varible to activate the Nesterov acceleration scheme.
@@ -1364,6 +1372,7 @@ class Grassmann:
 
         * **mean_element** (`list`)
             Karcher mean.
+
         """
 
         if 'tol' in kwargs.keys():
@@ -1447,6 +1456,7 @@ class Grassmann:
 
         * **frechet_var** (`list`)
             Frechet variance.
+
         """
         p_dim = []
         for i in range(len(points_grassmann)):
@@ -1475,21 +1485,7 @@ class Grassmann:
         matrices, if `point` and `samples` are matrices. The samples related to `coordinates` are set using
         `manifold`. For example, the following command is used to perform the interpolation.
 
-        >>> from UQpy.Surrogates import Kriging
-        >>> Krig = Kriging(reg_model='Linear', corr_model='Exponential', n_opt=1, dimension=2)
-        >>> Gr = Grassmann(distance_object=Grassmann.grassmann_distance,
-        >>>                interp_object=Krig, karcher_method=Grassmann.gradient_descent)
-        >>> Gr.manifold(p=p,samples=samples)
-        >>> interpolated = Gr.interpolate(coordinates=coordinates, point=point, element_wise=True)
-
         On the other hand, if a scikit learn gaussian_process object is provided, one can use the following commands:
-        
-        >>> from sklearn.gaussian_process import GaussianProcessRegressor
-        >>> gp = GaussianProcessRegressor()
-        >>> Gr = Grassmann(distance_method=Grassmann.grassmann_distance, 
-                           interp_object=gp,karcher_method=Grassmann.gradient_descent)
-        >>> Gr.manifold(p=p,samples=samples)
-        >>> interpolated = Gr.interpolate(coordinates=coordinates, point=[point], element_wise=True)
 
         **Input:**
 
@@ -1506,6 +1502,7 @@ class Grassmann:
 
         * **interpolated** (`list`)
             Interpolated point.
+
         """
 
         # Find the Karcher mean.
@@ -1568,6 +1565,7 @@ class Grassmann:
 
         * **interp_point** (`ndarray`)
             Interpolated point on the tangent space.
+
         """
 
         if isinstance(samples, list):
@@ -1667,6 +1665,7 @@ class Grassmann:
 
         * **interp_point** (`ndarray`)
             Interpolated point.
+
         """
 
         if not isinstance(coordinates, list) and not isinstance(coordinates, np.ndarray):
@@ -1738,9 +1737,9 @@ class DiffusionMaps:
           DiffusionMaps(kernel_object=DiffusionMaps.gaussian_kernel);
         - Using an user defined function as DiffusionMaps(kernel_object=user_kernel);
         - Passing a ``Grassmann`` class object DiffusionMaps(kernel_object=Grassmann_Object). In this case, the user has
-         to select ``kernel_grassmann`` in order to define which kernel matrix will be used because when the the
-         ``Grassmann`` class is used in a dataset a kernel matrix can be constructed with both the left and right
-         singular eigenvectors.
+          to select ``kernel_grassmann`` in order to define which kernel matrix will be used because when the the
+          ``Grassmann`` class is used in a dataset a kernel matrix can be constructed with both the left and right
+          singular eigenvectors.
 
     * **kernel_grassmann** (`str`)
         It assumes the values 'left' and 'right' for the left and right singular eigenvectors used to compute the kernel
@@ -1767,6 +1766,7 @@ class DiffusionMaps:
         Eigenvalues of the transition kernel of a Markov chanin on the data.
 
     **Methods:**
+
     """
 
     def __init__(self, alpha=0.5, n_evecs=2, sparse=False, k_neighbors=1, kernel_object=None, kernel_grassmann=None):
@@ -1814,19 +1814,11 @@ class DiffusionMaps:
         of input data points on the Grassmann manifold, or directly with the input data points. For example,
         considering that a ``Grassmann`` object is provided using the following command:
 
-        >>> Gr = Grassmann(kernel_method=Grassmann.projection_kernel)
-        >>> Gr.manifold(p=p,samples=samples,append_samples=False)
-
         one can instantiate the DiffusionMaps class and run the diffusion maps as follows:
-        >>> dfm = DiffusionMaps(alpha=0.5, n_evecs=10, kernel_object=Gr, kernel_grassmann = 'prod')
-        >>> diff_coords, evals, evecs = dfm.mapping()
 
         On the other hand, if the user wish to pass a dataset (samples) to compute the diffusion coordinates using the Gaussian
         kernel, one can use the following commands:
 
-        >>> dfm = DiffusionMaps(alpha=0.5, n_evecs=10, kernel_object=DiffusionMaps.gaussian_kernel)
-        >>> diff_coords, evals, evecs = dfm.mapping(samples=samples, epsilon=epsilon)
-
         In the latest case, if `epsilon` is not provided it is estimated based on the median of the square of the
         euclidian distances between data points.
 
@@ -1848,6 +1840,7 @@ class DiffusionMaps:
 
         * **evecs** (`ndarray`)
             eigenvectors.
+
         """
 
         alpha = self.alpha
@@ -1950,6 +1943,7 @@ class DiffusionMaps:
 
         * **Kernel matrix** (`ndarray`)
             Kernel matrix.
+
         """
 
         sparse = self.sparse
@@ -2013,6 +2007,7 @@ class DiffusionMaps:
 
         * **D_inv** (`list`)
             Inverse of matrix D.
+
         """
 
         nrows = np.shape(kernel_matrix)[0]
@@ -2051,6 +2046,7 @@ class DiffusionMaps:
 
         * **d_inv** (`list`)
             Inverse of matrix D.
+
         """
 
         d = np.array(kernel_matrix.sum(axis=1)).flatten()
@@ -2079,6 +2075,7 @@ class DiffusionMaps:
 
         * **normalized_kernel** (`list` or `ndarray`)
             Normalized kernel.
+
         """
 
         sparse = self.sparse
@@ -2091,3 +2088,428 @@ class DiffusionMaps:
         normalized_kernel = d_alpha.dot(kernel_mat.dot(d_alpha))
 
         return normalized_kernel
+
+
+########################################################################################################################
+########################################################################################################################
+#                                                     POD                                                              #
+########################################################################################################################
+########################################################################################################################
+
+
+class POD:
+    """
+    Performs Direct and Snapshot Proper Orthogonal Decomposition (POD) as well as Higher-order Singular Value
+    Decomposition (HOSVD) for dimension reduction of datasets.
+
+    **Input:**
+
+    * **input_sol** (`ndarray`) or (`list`):
+        Second order tensor or list containing the solution snapshots. Third dimension or length of list corresponds
+        to the number of snapshots.
+
+    * **verbose** (`Boolean`):
+        A boolean declaring whether to write text to the terminal.
+
+    **Methods:**
+    """
+
+    def __init__(self, input_sol, verbose=True, **kwargs):
+
+        self.input_sol = input_sol
+        self.verbose = verbose
+        self.kwargs = kwargs
+
+    @staticmethod
+    def unfold(data):
+        """
+        Method for unfolding second order tensors.
+
+        **Input:**
+
+        * **data** (`ndarray`) or (`list`):
+            Input second order tensor to be unfolded.
+
+        **Output/Returns:**
+
+        * **M0, M1, M2** (`ndarrays`):
+            Returns the 2-dimensional unfolded matrices.
+        """
+
+        if type(data) == list:
+            x, y, z = data[0].shape[0], data[0].shape[1], len(data)
+            data_ = np.zeros((x, y, z))
+            for i in range(z):
+                data_[:, :, i] = data[i]
+            del data
+            data = np.copy(data_)
+
+        d0, d1, d2 = [0, 2, 1], [1, 2, 0], [2, 0, 1]
+        z0, z1, z2 = np.transpose(data, d0), np.transpose(data, d1), np.transpose(data, d2)
+
+        m0 = z0.reshape(data.shape[0], data.shape[2] * data.shape[1])
+        m1 = z1.reshape(data.shape[1], data.shape[2] * data.shape[0])
+        m2 = z2.reshape(data.shape[2], data.shape[0] * data.shape[1])
+
+        return m0, m1, m2
+
+
+class DirectPOD(POD):
+    """
+    Direct POD child class generates a set of spatial modes and time coefficients to approximate the solution.
+
+    **Input:**
+
+    * **input_sol** (`ndarray`) or (`list`):
+        Second order tensor or list containing the solution snapshots. Third dimension or length of list corresponds
+        to the number of snapshots.
+
+    * **modes** (`int`):
+        Number of POD modes used to approximate the input solution. Must be less than or equal
+        to the number of grid points.
+
+    * **reconstr_perc** (`float`):
+        Dataset reconstruction percentage.
+
+    **Methods:**
+    """
+
+    def __init__(self, input_sol, modes=10**10, reconstr_perc=10**10, verbose=False):
+
+        super().__init__(input_sol, verbose)
+        self.verbose = verbose
+        self.modes = modes
+        self.reconstr_perc = reconstr_perc
+
+    def run(self):
+
+        """
+        Executes the Direct POD method in the ''Direct'' class.
+
+        **Output/Returns:**
+
+        * **reconstructed_solutions** (`ndarray`):
+            Second order tensor containing the reconstructed solution snapshots in their initial spatial and
+            temporal dimensions.
+
+        * **reduced_solutions** (`ndarray`):
+            An array containing the solution snapshots reduced in the spatial dimension.
+
+        """
+
+        if type(self.input_sol) == list:
+
+            x, y, z = self.input_sol[0].shape[0], self.input_sol[0].shape[1], len(self.input_sol)
+            u = np.zeros((z, x * y))
+
+            for i in range(z):
+                u[i, :] = self.input_sol[i].ravel()
+
+        else:
+            x, y, z = self.input_sol.shape[0], self.input_sol.shape[1], self.input_sol.shape[2]
+            u = np.zeros((z, x * y))
+
+            for i in range(z):
+                u[i, :] = self.input_sol[:, :, i].ravel()
+
+        c = np.dot(u.T, u) / (z - 1)
+        eigval, phi = np.linalg.eig(c)
+        phi = phi.real
+        eigval_ = eigval.real
+        a = np.dot(u, phi)
+
+        if self.modes <= 0:
+            print('Warning: Invalid input, the number of modes must be positive.')
+            return [], []
+
+        elif self.reconstr_perc <= 0:
+            print('Warning: Invalid input, the reconstruction percentage is defined in the range (0,100].')
+            return [], []
+
+        elif self.modes != 10**10 and self.reconstr_perc != 10**10:
+            print('Warning: Either a number of modes or a reconstruction percentage must be chosen, not both.')
+            return [], []
+
+        elif type(self.modes) != int:
+            print('Warning: The number of modes must be an integer.')
+            return [], []
+
+        else:
+
+            perc = []
+            for i in range(x * y):
+                perc.append((eigval_[:i + 1].sum() / eigval_.sum()) * 100)
+
+            percentage = min(perc, key=lambda x: abs(x - self.reconstr_perc))
+
+            if self.modes == 10**10:
+
+                self.modes = perc.index(percentage) + 1
+
+            else:
+
+                if self.modes > x * y:
+                    print("Warning: A number of modes greater than the number of dimensions was given.")
+                    print("Number of dimensions is {}".format(x * y))
+
+            reconstructed_solutions_ = np.dot(a[:, :self.modes], phi[:, :self.modes].T)
+            reduced_solutions = np.dot(u, phi[:, :self.modes])
+
+            reconstructed_solutions = np.zeros((x, y, z))
+            for i in range(z):
+                reconstructed_solutions[0:x, 0:y, i] = reconstructed_solutions_[i, :].reshape((x, y))
+
+            if self.verbose:
+                print("UQpy: Successful execution of Direct POD!")
+                if z < x * y and x * y > 1000:
+                    print("Snapshot POD is recommended.")
+
+            if self.verbose:
+                print('Dataset reconstruction: {:.3%}'.format(perc[self.modes - 1] / 100))
+
+            return reconstructed_solutions, reduced_solutions
+
+
+class SnapshotPOD(POD):
+    """
+    Snapshot POD child class generates a set of temporal modes and spatial coefficients to approximate the solution.
+    (Faster that direct POD)
+
+    **Input:**
+
+    * **input_sol** (`ndarray`) or (`list`):
+        Second order tensor or list containing the solution snapshots. Third dimension or length of list corresponds
+        to the number of snapshots.
+
+    * **modes** (`int`):
+        Number of POD modes used to approximate the input solution. Must be less than or equal
+        to the number of grid points.
+
+    * **reconstr_perc** (`float`):
+        Dataset reconstruction percentage.
+
+    **Methods:**
+   """
+
+    def __init__(self, input_sol, modes=10**10, reconstr_perc=10**10, verbose=False):
+
+        super().__init__(input_sol, verbose)
+        self.verbose = verbose
+        self.modes = modes
+        self.reconstr_perc = reconstr_perc
+
+    def run(self):
+        """
+        Executes the Snapshot POD method in the ''Snapshot'' class.
+
+        **Output/Returns:**
+
+        * **reconstructed_solutions** (`ndarray`):
+            Second order tensor containing the reconstructed solution snapshots in their initial spatial and
+            temporal dimensions.
+
+        * **reduced_solutions** (`ndarray`):
+            An array containing the solution snapshots reduced in the temporal dimension.
+
+        """
+        if type(self.input_sol) == list:
+
+            x, y, z = self.input_sol[0].shape[0], self.input_sol[0].shape[1], len(self.input_sol)
+            u = np.zeros((z, x * y))
+
+            for i in range(z):
+                u[i, :] = self.input_sol[i].ravel()
+
+        else:
+            x, y, z = self.input_sol.shape[0], self.input_sol.shape[1], self.input_sol.shape[2]
+            u = np.zeros((z, x * y))
+
+            for i in range(z):
+                u[i, :] = self.input_sol[:, :, i].ravel()
+
+        c_s = np.dot(u, u.T) / (z - 1)
+
+        eigval, a_s = np.linalg.eig(c_s)
+        a_s = a_s.real
+        eigval_ = eigval.real
+
+        if self.modes <= 0:
+            print('Warning: Invalid input, the number of modes must be positive.')
+            return [], []
+
+        elif self.reconstr_perc <= 0:
+            print('Warning: Invalid input, the reconstruction percentage is defined in the range (0,100].')
+            return [], []
+
+        elif self.modes != 10**10 and self.reconstr_perc != 10**10:
+            print('Warning: Either a number of modes or a reconstruction percentage must be chosen, not both.')
+            return [], []
+
+        elif type(self.modes) != int:
+            print('Warning: The number of modes must be an integer.')
+            return [], []
+
+        else:
+
+            perc = []
+            for i in range(z):
+                perc.append((eigval_[:i + 1].sum() / eigval_.sum()) * 100)
+
+            percentage = min(perc, key=lambda x: abs(x - self.reconstr_perc))
+
+            if self.modes == 10**10:
+
+                self.modes = perc.index(percentage) + 1
+
+            else:
+
+                if self.modes > z:
+                    print("Warning: A number of modes greater than the number of dimensions was given.")
+                    print("Number of dimensions is {}".format(z))
+
+            phi_s = np.dot(u.T, a_s)
+            reconstructed_solutions_ = np.dot(a_s[:, :self.modes], phi_s[:, :self.modes].T)
+            reduced_solutions_ = (np.dot(u.T, a_s[:, :self.modes])).T
+
+            reconstructed_solutions = np.zeros((x, y, z))
+            reduced_solutions = np.zeros((x, y, self.modes))
+
+            for i in range(z):
+                reconstructed_solutions[0:x, 0:y, i] = reconstructed_solutions_[i, :].reshape((x, y))
+
+            for i in range(self.modes):
+                reduced_solutions[0:x, 0:y, i] = reduced_solutions_[i, :].reshape((x, y))
+
+            if self.verbose:
+                print("UQpy: Successful execution of Snapshot POD!")
+
+            if self.verbose:
+                print('Dataset reconstruction: {:.3%}'.format(perc[self.modes - 1] / 100))
+
+            return reconstructed_solutions, reduced_solutions
+
+
+class HOSVD(POD):
+    """
+    HOSVD child class is used for higher-order singular value decomposition on the input solutions tensor.
+
+    **Inputs:**
+
+    * **input_sol** (`ndarray`) or (`list`):
+        Second order tensor or list containing the solution snapshots. Third dimension or length of list corresponds
+        to the number of snapshots.
+
+    * **modes** (`int`):
+        Number of modes to keep for dataset reconstruction.
+
+    * **reconstr_perc** (`float`):
+        Dataset reconstruction percentage
+
+    **Methods:**
+    """
+
+    def __init__(self, input_sol, modes=10**10, reconstr_perc=10**10, verbose=False):
+
+        super().__init__(input_sol, verbose)
+        self.verbose = verbose
+        self.modes = modes
+        self.reconstr_perc = reconstr_perc
+
+    def run(self, get_error=False):
+        """
+        Executes the HOSVD method in the ''HOSVD'' class.
+
+        **Input:**
+
+        * **get_error** (`Boolean`):
+            A boolean declaring whether to return the reconstruction error.
+
+        **Output/Returns:**
+
+        * **reconstructed_solutions** (`ndarray`):
+            Second order tensor containing the reconstructed solution snapshots in their initial spatial and
+            temporal dimensions.
+
+        * **reduced_solutions** (`ndarray`):
+            An array containing the reduced solutions snapshots. The array's dimensions depends on the dimensions
+            of input second order tensor and not on the input number of modes or reconstruction percentage.
+
+        """
+
+        if type(self.input_sol) == list:
+            x, y, z = self.input_sol[0].shape[0], self.input_sol[0].shape[1], len(self.input_sol)
+        else:
+            x, y, z = self.input_sol.shape[0], self.input_sol.shape[1], self.input_sol.shape[2]
+
+        a1, a2, a3 = POD.unfold(self.input_sol)
+
+        u1, sig_1, v1 = np.linalg.svd(a1, full_matrices=True)
+        u2, sig_2, v2 = np.linalg.svd(a2, full_matrices=True)
+        u3, sig_3, v3 = np.linalg.svd(a3, full_matrices=True)
+
+        sig_3_ = np.diag(sig_3)
+        hold = np.dot(np.linalg.inv(u3), a3)
+        kron_ = np.kron(u1, u2)
+
+        s3 = np.array(np.dot(hold, np.linalg.inv(kron_.T)))
+
+        if self.modes <= 0:
+            print('Warning: Invalid input, the number of modes must be positive.')
+            return [], []
+
+        elif self.reconstr_perc <= 0:
+            print('Warning: Invalid input, the reconstruction percentage is defined in the range (0,100].')
+            return [], []
+
+        elif self.modes != 10**10 and self.reconstr_perc != 10**10:
+            print('Warning: Either a number of modes or a reconstruction percentage must be chosen, not both.')
+            return [], []
+
+        elif type(self.modes) != int:
+            print('Warning: The number of modes must be an integer.')
+            return [], []
+
+        else:
+
+            if self.modes == 10**10:
+                error_ = []
+                for i in range(0, x):
+                    error_.append(np.sqrt(((sig_3[i + 1:]) ** 2).sum()) / np.sqrt((sig_3 ** 2).sum()))
+                    if i == x:
+                        error_.append(0)
+                error = [i * 100 for i in error_]
+                error.reverse()
+                perc = error.copy()
+                percentage = min(perc, key=lambda x: abs(x-self.reconstr_perc))
+                self.modes = perc.index(percentage) + 1
+
+            else:
+                if self.modes > x:
+                    print("Warning: A number of modes greater than the number of temporal dimensions was given.")
+                    print("Number of temporal dimensions is {}.".format(x))
+
+            reduced_solutions = np.dot(u3, sig_3_)
+            u3hat = np.dot(u3[:, :self.modes], sig_3_[:self.modes, :self.modes])
+            s3hat = np.dot(np.linalg.inv(sig_3_[:self.modes, :self.modes]), s3[:self.modes, :])
+
+            b = np.kron(u1, u2)
+            c = np.dot(s3hat, b.T)
+            d = np.dot(u3hat[:, :], c)
+
+            reconstructed_solutions = np.zeros((x, y, z))
+            for i in range(z):
+                reconstructed_solutions[0:x, 0:y, i] = d[i, :].reshape((x, y))
+
+            if self.verbose:
+                print("UQpy: Successful execution of HOSVD!")
+
+            if self.modes == 10**10:
+                if self.verbose:
+                    print('Dataset reconstruction: {0:.3%}'.format(percentage / 100))
+
+            else:
+                if get_error:
+                    error_rec = np.sqrt(((sig_3[self.modes:]) ** 2).sum()) / np.sqrt((sig_3 ** 2).sum())
+                    print("Reduced-order reconstruction error: {0:.3%}".format(error_rec))
+
+            return reconstructed_solutions, reduced_solutions
